--[[ 
ToDo list
    Ask for interrupt spells after a reload (low)
    Leaving a full party didn't clear all bars
]]
----------------------------------------
-- local variables and functions
----------------------------------------

-- variables passed to this file from the game
local addonName, ZI = ...

-- the main object for the addon is a frame
if not _G[addonName] then
    _G[addonName] = CreateFrame("Frame", addonName, UIParent, BackdropTemplateMixin and "BackdropTemplate")
end
ZI = _G[addonName]

-- redirect events to functions on the main object using the event name
ZI:SetScript(
    "OnEvent",
    function(self, event, ...)
        if self[event] then
            return self[event](self, event, ...)
        end
    end
)

-- setup a local debugging function that will use ViragDevTool when it's available
local debugf = false
local function Debug(...)
    if debugf then
        if (ViragDevTool_AddData) then
            if (type(...) == "table") then
                ViragDevTool_AddData(..., addonName)
            else
                local debugMessage = string.join(", ", tostringall(...))
                ViragDevTool_AddData(debugMessage, addonName)
            end
        else
            local debugMessage = string.join(", ", tostringall(...))
            DEFAULT_CHAT_FRAME:AddMessage(addonName .. ":" .. debugMessage)
        end
    end
end

----------------------------------------
-- Main object definitions
----------------------------------------
ZI.PlayerName = ""
ZI.InGroup = false
ZI.UnitInterrupts = {}
ZI.UnitInterruptOrder = {}
ZI.StatusBars = {}
ZI.PlayerInterrupts = {}

function ZI:OnUpdate(elapsed)
    for k, v in pairs(ZI.StatusBars) do
        local unitInterrupt = ZI.UnitInterruptOrder[k]
        if (not unitInterrupt or unitInterrupt.Available == false) then
            v:SetValue(0)
        elseif (not unitInterrupt.Ready) then
            local newValue = unitInterrupt.ReadyTime - GetTime()
            if (newValue <= 0) then
                v:SetValue(0)
                unitInterrupt.Ready = true
            else
                v:SetValue(newValue)
            end
        else
            v:SetValue(0)
        end
    end
end

function ZI:CombatLogEvent(...)
    local _, event, _, sourceGUID, sourceName, _, _, _, _, _, _, spellId, spellName = CombatLogGetCurrentEventInfo()

    if not (sourceName == ZI.PlayerName or UnitInParty(sourceName) or UnitInRaid(sourceName)) then
        return
    end

    if (event == "SPELL_CAST_SUCCESS") then
        if (tContains(ZI.InterruptSpells.SpellId, spellId)) then
            -- If an interrupt spell was cast
            ZI:InterruptUsed(sourceName, spellId)
        end
    end
end

function ZI:GetUnitInterruptKey(unitName, spellId)
    return unitName .. "-" .. tostring(spellId)
end

function ZI:InterruptUsed(sourceName, spellId)
    local unitInterrupt = ZI.UnitInterrupts[ZI:GetUnitInterruptKey(sourceName, spellId)]
    if (not unitInterrupt) then
        -- add this unit spell to the list
        -- make sure the player is in the party??
        local interruptCooldown, interruptName = ZI:GetInterruptInfo(spellId)
        ZI:AddOrUpdateUnitInterrupt(sourceName, spellId, interruptCooldown, interruptName)
        unitInterrupt = ZI.UnitInterrupts[ZI:GetUnitInterruptKey(sourceName, spellId)]
    end
    if (unitInterrupt) then
        unitInterrupt.ReadyTime = GetTime() + unitInterrupt.Cooldown
        unitInterrupt.Ready = false
        ZI:UpdateInterruptRotation()
        if (ZI.InGroup) then
            local nextInLine = ZI.UnitInterruptOrder[1]
            ZI:ShowMessage(nextInLine.UnitName, 2, 1, 0, 0)
            if (nextInLine and nextInLine.IsPlayer) then
                ZI:PlaySoundFile()
            end
        end
    end
end

function ZI:PlaySoundFile()
    if (ZoomInterruptsDB.OggFileName) then
        if (ZoomInterruptsDB.OggFileName ~= "None") then
            local soundFile = "Interface\\AddOns\\ZoomInterrupts\\Media\\" .. ZoomInterruptsDB.OggFileName .. ".ogg"
            PlaySoundFile(soundFile)
        end
    end
end

function ZI:SendAddonMessage(msg)
    --Debug("ZI:SendAddonMessage(msg)")
    --Debug(msg)
    C_ChatInfo.SendAddonMessage(addonName, msg, "PARTY")
end

function ZI:AddonMessageReceived(text, channel, sender, target, zoneChannelID, localID, name, instanceID)
    --Debug("ZI:AddonMessageReceived()")
    --Debug(text)
    local results = {}
    for match in string.gmatch(text, "[^,]+") do
        table.insert(results, match)
    end
    local sourceName = results[1]
    local spellId = tonumber(results[2])
    --Debug(sourceName)
    --Debug(spellId)
    local unitInterrupt = ZI.UnitInterrupts[ZI:GetUnitInterruptKey(sourceName, spellId)]
    if (not unitInterrupt) then
        -- add this unit spell to the list
        -- make sure the player is in the party??
        local interruptCooldown, interruptName = ZI:GetInterruptInfo(spellId)
        ZI:AddOrUpdateUnitInterrupt(sourceName, spellId, interruptCooldown, interruptName)
        unitInterrupt = ZI.UnitInterrupts[ZI:GetUnitInterruptKey(sourceName, spellId)]
    end
end

function ZI:CreateTextFrame()
    local f = CreateFrame("MessageFrame", addonName .. "MessageFrame", UIParent)
    ZI.MessageFrame = f
    f:SetSize(500, 50)
    f:SetPoint("CENTER", UIParent, "CENTER")
    f:SetClampedToScreen(true)

    f:SetFontObject(BossEmoteNormalHuge)

    local t = f:CreateTexture()
    f.texture = t
    t:SetAllPoints(f)

    local fontPath = f:GetFont()
    f:SetFont(fontPath, 45, "OUTLINE")
    f:SetFadeDuration(0.4)
end

function ZI:CreateMainFrame()
    if (UnitInParty("player") or UnitInRaid("player")) then
        ZI.InGroup = true
    end
    self:SetPoint("CENTER", UIParent, "CENTER")
    self:SetSize(120, 170)
    self:SetBackdrop(
        {
            bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
            edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
            tile = false,
            tileSize = 1,
            edgeSize = 10,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    self:SetBackdropColor(0, 0, 0, 0.75)
    self:EnableMouse(true)
    self:SetMovable(true)
    -- self:SetResizable(true)
    -- self:SetMinResize(120, 150)
    self:SetClampedToScreen(true)
    self:SetScale(1)
    ZI:RestoreLayout(addonName)

    self:SetScript(
        "OnMouseDown",
        function()
            self.isMoving = true
            self:StartMoving()
        end
    )

    self:SetScript(
        "OnMouseUp",
        function()
            if (self.isMoving) then
                self.isMoving = nil
                self:StopMovingOrSizing()
                ZI:SaveLayout(addonName)
            end
        end
    )

    self:Hide()

    -- local resizeButton = CreateFrame("Button", nil, self)
    -- resizeButton:SetSize(16, 16)
    -- resizeButton:SetPoint("BOTTOMRIGHT")
    -- resizeButton:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up")
    -- resizeButton:SetHighlightTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Highlight")
    -- resizeButton:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Down")

    -- resizeButton:SetScript("OnMouseDown", function()
    --     self:StartSizing("BOTTOMRIGHT")
    -- end)
    -- resizeButton:SetScript("OnMouseUp", function()
    --     self:StopMovingOrSizing()
    -- end)

    ZI.OptionNext = ZI:CreateCheckbutton(10, -10, "Next")
    ZI.OptionOwl = ZI:CreateCheckbutton(10, -30, "Owl")
    ZI.OptionWaterDrop = ZI:CreateCheckbutton(10, -50, "WaterDrop")
    ZI.OptionNone = ZI:CreateCheckbutton(10, -70, "(no sound)")

    Debug("LoadOptions")
    ZI:LoadOptions()

    ZI.OptionNext:SetScript(
        "OnClick",
        function()
            ZI:UpdateOptions("Next")
        end
    )

    ZI.OptionOwl:SetScript(
        "OnClick",
        function()
            ZI:UpdateOptions("Owl")
        end
    )

    ZI.OptionWaterDrop:SetScript(
        "OnClick",
        function()
            ZI:UpdateOptions("WaterDrop")
        end
    )

    ZI.OptionNone:SetScript(
        "OnClick",
        function()
            ZI:UpdateOptions("None")
        end
    )

    local playButton = ZI:CreateButton(33, -96, "Play")
    playButton:SetScript(
        "OnClick",
        function(self)
            ZI:PlaySoundFile()
        end
    )

    local closeButton = ZI:CreateButton(30, -128, "Close")
    closeButton:SetScript(
        "OnClick",
        function(self)
            ZI:CloseClick()
        end
    )

    ZI:CreateTextFrame()
    ZI:CreateStatusBarAnchor()
    ZI:CreateStatusBars()
    ZI:UpdateInterruptRotation()
end

function ZI:LoadOptions()
    if (not ZoomInterruptsDB.OggFileName) then
        ZoomInterruptsDB.OggFileName = "Next"
    end
    ZI:ResetOptions()
    ZI["Option" .. ZoomInterruptsDB.OggFileName]:SetChecked(true)
end

function ZI:ResetOptions()
    ZI.OptionNext:SetChecked(false)
    ZI.OptionOwl:SetChecked(false)
    ZI.OptionWaterDrop:SetChecked(false)
    ZI.OptionNone:SetChecked(false)
end

function ZI:UpdateOptions(selectedOption)
    ZI:ResetOptions()
    ZI["Option" .. selectedOption]:SetChecked(true)
    ZoomInterruptsDB.OggFileName = selectedOption
end

function ZI:GetTipAnchor(frame)
    local x, y = frame:GetCenter()
    if not x or not y then
        return "TOPLEFT", "BOTTOMLEFT"
    end
    local hhalf = (x > UIParent:GetWidth() * 2 / 3) and "RIGHT" or (x < UIParent:GetWidth() / 3) and "LEFT" or ""
    local vhalf = (y > UIParent:GetHeight() / 2) and "TOP" or "BOTTOM"
    return vhalf .. hhalf, frame, (vhalf == "TOP" and "BOTTOM" or "TOP") .. hhalf
end

function ZI:ClickMeClick()
    --
end

function ZI:CloseClick()
    ZI:Hide()
end

function ZI:AddOrUpdateUnitInterrupt(unitName, interruptId, interruptCooldown, interruptName)
    local unitInterruptKey = ZI:GetUnitInterruptKey(unitName, interruptId)
    local unitInterrupt = ZI.UnitInterrupts[unitInterruptKey]
    if (not unitInterrupt) then
        local unitInterrupt = {}
        unitInterrupt.UnitName = unitName
        unitInterrupt.IsPlayer = unitName == ZI.PlayerName
        unitInterrupt.InterruptId = interruptId
        unitInterrupt.InterruptName = interruptName
        unitInterrupt.StatusBarText = unitName .. "-" .. interruptName
        unitInterrupt.Cooldown = interruptCooldown
        unitInterrupt.Ready = true
        unitInterrupt.ReadyTime = GetTime()
        unitInterrupt.Offline = false
        unitInterrupt.Dead = false
        unitInterrupt.Available = true
        ZI.UnitInterrupts[unitInterruptKey] = unitInterrupt
        table.insert(ZI.UnitInterruptOrder, unitInterrupt)
    end
    ZI:UpdateInterruptRotation()
end

function ZI:ShowMessage(text, secondsVisible, r, g, b)
    ZI.MessageFrame:AddMessage(text, r, g, b)
    ZI.MessageFrame:SetTimeVisible(secondsVisible)
    ZI.MessageFrame.text = text
end

function ZI:CreateButton(x_loc, y_loc, displayText)
    if (not ZI.ButtonIndex) then
        ZI.ButtonIndex = 0
    end
    ZI.ButtonIndex = ZI.ButtonIndex + 1

    local button = CreateFrame("Button", addonName .. "_button_" .. ZI.ButtonIndex, ZI)

    button:SetText(displayText)
    button:SetNormalFontObject("GameFontNormal")

    button:SetPoint("TOPLEFT", ZI, "TOPLEFT", x_loc, y_loc)

    button:SetWidth(button:GetTextWidth() + 30)
    button:SetHeight(30)

    local ntex = button:CreateTexture()
    ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
    ntex:SetTexCoord(0, 0.625, 0, 0.6875)
    ntex:SetAllPoints()
    button:SetNormalTexture(ntex)

    local htex = button:CreateTexture()
    htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
    htex:SetTexCoord(0, 0.625, 0, 0.6875)
    htex:SetAllPoints()
    button:SetHighlightTexture(htex)

    local ptex = button:CreateTexture()
    ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
    ptex:SetTexCoord(0, 0.625, 0, 0.6875)
    ptex:SetAllPoints()
    button:SetPushedTexture(ptex)

    return button
end

function ZI:CreateCheckbutton(x_loc, y_loc, displayText)
    if (not ZI.CheckIndex) then
        ZI.CheckIndex = 0
    end
    ZI.CheckIndex = ZI.CheckIndex + 1

    Debug(addonName .. "_checkbutton_" .. ZI.CheckIndex)
    local checkbutton =
        CreateFrame("CheckButton", addonName .. "_chkbtn_" .. ZI.CheckIndex, ZI, "ChatConfigCheckButtonTemplate")
    checkbutton:SetPoint("TOPLEFT", x_loc, y_loc)
    getglobal(checkbutton:GetName() .. "Text"):SetText(" " .. displayText)

    return checkbutton
end

function ZI:CreateStatusBarAnchor()
    local f =
        CreateFrame("Frame", addonName .. "StatusBarAnchor", UIParent, BackdropTemplateMixin and "BackdropTemplate")
    ZI[addonName .. "StatusBarAnchor"] = f
    f:SetPoint("CENTER", UIParent, "CENTER")
    f:SetSize(220, 100)
    f:SetBackdropColor(0, 0, 0, 0.70)
    f:SetMovable(true)
    f:SetClampedToScreen(true)

    ZI:RestoreLayout(addonName .. "StatusBarAnchor")
    ZI:RepositionMessageFrame(f)

    f:SetScript(
        "OnMouseDown",
        function(self, button)
            if (IsShiftKeyDown() and IsAltKeyDown()) then
                f.isMoving = true
                f:StartMoving()
            elseif (IsAltKeyDown()) then
                ZI:Show()
            else
                local fontColor = {r = 1, g = 210 / 255, b = 0}

                GameTooltip:SetOwner(self, "ANCHOR_NONE")
                GameTooltip:SetPoint(ZI:GetTipAnchor(self))

                GameTooltip:ClearLines()
                GameTooltip:AddLine(addonName)
                GameTooltip:AddDoubleLine(
                    "Shift-Alt-Click",
                    "Drag frame to new location",
                    129 / 255,
                    209 / 255,
                    92 / 255,
                    1,
                    1,
                    1
                )
                GameTooltip:AddDoubleLine("Alt-Click", "Open options panel", 129 / 255, 209 / 255, 92 / 255, 1, 1, 1)

                GameTooltip:Show()
            end
        end
    )

    f:SetScript(
        "OnMouseUp",
        function()
            if (f.isMoving) then
                f.isMoving = nil
                f:StopMovingOrSizing()
                ZI:SaveLayout(addonName .. "StatusBarAnchor")
                ZI:RepositionMessageFrame(f)
            end
        end
    )

    f:SetScript(
        "OnLeave",
        function()
            GameTooltip:Hide()
        end
    )

    f:SetScript(
        "OnUpdate",
        function(self, elapsed)
            ZI:OnUpdate(elapsed)
        end
    )

    t = f:CreateTexture()
    t:SetAllPoints(f)
    ZI[addonName .. "StatusBarColorTexture"] = t

    ZI:SetStatusBarBackColor("G")
end

function ZI:RepositionMessageFrame(f)
    local point, relativeTo, relativePoint, xOfs, yOfs = f:GetPoint()
    ZI.MessageFrame:ClearAllPoints()
    ZI.MessageFrame:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs + 75)
end

function ZI:SetStatusBarBackColor(colorToSet)
    local statusBarColorTexture = ZI[addonName .. "StatusBarColorTexture"]
    if (not statusBarColorTexture) then
        return
    end
    if (not ZI.BackColor) then
        ZI.BackColor = ""
    end
    if (colorToSet == "R" and ZI.BackColor ~= "R") then
        statusBarColorTexture:SetColorTexture(1, 0, 0, 0.2)
        ZI.BackColor = "R"
    elseif (colorToSet == "G" and ZI.BackColor ~= "G") then
        statusBarColorTexture:SetColorTexture(0, 1, 0, 0.1)
        ZI.BackColor = "G"
    end
end

function ZI:CreateFontString(name, parent, flags, supportCyrillic)
    local t = parent:CreateFontString(name)
    t.font = ""
    t.flags = ""

    t:SetFont("Fonts\\FRIZQT___CYR.TTF", 12, flags)
    t.font = "Fonts\\FRIZQT___CYR.TTF"

    function t:SetFontSize(size)
        self:SetFont(t.font, size, flags)
    end

    t:SetTextColor(1, 1, 1)
    return t
end

function ZI:CreateStatusBars()
    local statusBars = ZI.StatusBars
    local numBars = 5
    local statusBarAlpha = 1
    local statusBarTextAlpha = 1
    local iconAlpha = 1
    local iconTextAlpha = 1

    for i = 1, numBars do
        if (not ZI.StatusBars[i]) then
            local f = CreateFrame("StatusBar", addonName .. "StatusBar" .. i, ZI[addonName .. "StatusBarAnchor"])
            local width, height = f:GetParent():GetSize()
            local fontSize = 0.58 * height / numBars
            local barHeight = height / numBars
            f:SetOrientation("HORIZONTAL")
            f:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
            f:SetStatusBarColor(0, 1, 0, statusBarAlpha)
            f:SetFrameLevel(3)
            f:SetID(i)
            f:SetMinMaxValues(0, 1)
            f:SetValue(0)
            statusBars[i] = f

            f.text = ZI:CreateFontString(nil, f, nil, true)
            f.text:SetPoint("LEFT", f, "LEFT", 5, 0)
            f.text:SetJustifyH("LEFT")
            f.text:SetTextColor(1, 1, 1, statusBarTextAlpha)

            f.cooldownText = ZI:CreateFontString(nil, f, "OUTLINE")
            f.cooldownText:SetPoint("RIGHT", f, "RIGHT", 3, 0)
            f.cooldownText:SetJustifyH("LEFT")
            f.cooldownText:SetTextColor(1, 1, 1, statusBarTextAlpha)

            local g = CreateFrame("Frame", addonName .. "Icon" .. i, f)
            f.icon = g
            g:SetPoint("RIGHT", f, "LEFT")

            local t = g:CreateTexture()
            t:SetAllPoints(g)
            t:SetColorTexture(0, 0, 0, iconAlpha)

            local t = ZI:CreateFontString(nil, g)
            g.text = t
            t:SetAllPoints(g)
            t:SetText(i)
            t:SetTextColor(1, 1, 1, iconTextAlpha)

            local function OnValueChanged(self, value)
                if (value <= 0) then
                    self.cooldownText:SetText("")
                else
                    self.cooldownText:SetText(ZI:RoundString(value, 1))
                end
            end

            f:SetScript("OnValueChanged", OnValueChanged)

            function f:UpdateSize()
                if (numBars == 0) then
                    return
                end

                local width, height = self:GetParent():GetSize()
                local fontSize = 0.62 * height / numBars
                local barHeight = height / numBars
                local iconWidth = barHeight

                if (iconWidth > width / 7) then
                    iconWidth = width / 7
                end

                if (fontSize > width / 15) then
                    fontSize = width / 15
                end

                self:SetSize(width - iconWidth, barHeight)
                self:SetPoint(
                    "TOPRIGHT",
                    ZI[addonName .. "StatusBarAnchor"],
                    "TOPRIGHT",
                    0,
                    (1 - self:GetID()) * barHeight
                )

                self.text:SetSize(width - 5, barHeight)
                self.text:SetFontSize(fontSize)

                self.cooldownText:SetSize(2 * barHeight, barHeight)
                self.cooldownText:SetFontSize(fontSize)

                self.icon:SetSize(iconWidth, barHeight)
                self.icon.text:SetFontSize(fontSize)
            end

            f:UpdateSize()
        else
            statusBars[i]:Show()
        end
    end
end

function ZI:SetInterruptSpells()
    ZI.InterruptSpells = {
        -- spellNames (The first Spell Lock ID 132409, is the spell gained from Grimoire of Sacrifice)
        ["SpellName"] = {
            "Spell Lock",
            "Optical Blast",
            "Spear Hand Strike",
            "Counter Shot",
            "Rebuke",
            "Skull Bash",
            "Solar Beam",
            "Mind Freeze",
            "Counterspell",
            "Kick",
            "Wind Shear",
            "Spell Lock",
            "Pummel",
            "Silence",
            "Disrupt",
            "Call Felhunter",
            "Muzzle"
        },
        -- spellIds
        ["SpellId"] = {
            132409,
            119911,
            116705,
            147362,
            96231,
            106839,
            78675,
            47528,
            2139,
            1766,
            57994,
            119910,
            6552,
            15487,
            183752,
            212619,
            187707
        },
        -- spellCooldowns
        ["Cooldown"] = {24, 24, 15, 24, 15, 15, 60, 15, 24, 15, 12, 24, 15, 45, 15, 24, 15}
    }
end

function ZI:GetInterruptInfo(interruptId)
    for tbSpellPos, tbSpellId in pairs(ZI.InterruptSpells.SpellId) do
        if (tbSpellId == interruptId) then
            return ZI.InterruptSpells.Cooldown[tbSpellPos], ZI.InterruptSpells.SpellName[tbSpellPos]
        end
    end
    return nil, nil
end

function ZI:RoundString(num, dec)
    return string.format("%." .. dec .. "f", tostring(num))
end

function ZI:UpdateInterruptRotation()
    for _, unitInterrupt in pairs(ZI.UnitInterrupts) do
        if (unitInterrupt) then
            if (ZI:IsUnitOffline(unitInterrupt.UnitName)) then
                unitInterrupt.Offline = true
            else
                unitInterrupt.Offline = false
            end

            if (ZI:IsUnitDead(unitInterrupt.UnitName)) then
                unitInterrupt.Dead = true
            else
                unitInterrupt.Dead = false
            end

            if (unitInterrupt.Offline or unitInterrupt.Dead) then
                unitInterrupt.Available = false
            else
                unitInterrupt.Available = true
            end
        end
    end

    table.sort(
        ZI.UnitInterruptOrder,
        function(a, b)
            return ZI:SortFunction(a, b)
        end
    )

    -- Update the UI to reflect the updated rotation
    for k, statusBar in pairs(ZI.StatusBars) do
        local unitInterrupt = ZI.UnitInterruptOrder[k]
        if (not unitInterrupt) then
            statusBar.text:SetText("")
        else
            if (unitInterrupt.Offline) then
                statusBar.text:SetText("(Offline) " .. unitInterrupt.StatusBarText)
            elseif (unitInterrupt.Dead) then
                statusBar.text:SetText("(Dead) " .. unitInterrupt.StatusBarText)
            else
                statusBar.text:SetText(unitInterrupt.StatusBarText)
            end
            statusBar:SetMinMaxValues(0, unitInterrupt.Cooldown)
        end
    end

    -- set the color
    local unitInterrupt = ZI.UnitInterruptOrder[1]
    --Debug("Top Interrupter=" .. unitInterrupt.UnitName .. " IsPlayer=" .. tostring(unitInterrupt.IsPlayer))
    if (unitInterrupt and unitInterrupt.IsPlayer) then
        ZI:SetStatusBarBackColor("G")
    else
        ZI:SetStatusBarBackColor("R")
    end
end

function ZI:IsUnitOffline(unit)
    if (not UnitExists(unit) or not UnitIsConnected(unit)) then
        return true
    else
        return false
    end
end

function ZI:IsUnitDead(unit)
    if (unit and UnitIsDead(unit)) then
        return true
    else
        return false
    end
end

function ZI:SortFunction(a, b)
    -- Returns true if a should be placed before b
    if (not a) then
        return false
    end
    if (not b) then
        return true
    end
    if (a.Available and b.Available) then
        if (a.Ready and b.Ready) then
            return a.Cooldown < b.Cooldown
        elseif (not a.Ready and not b.Ready) then
            return a.ReadyTime < b.ReadyTime
        elseif (a.Ready) then
            return true
        else
            return false
        end
    elseif (not a.Available and not b.Available) then
        return a.UnitName < b.UnitName
    elseif (a.Available) then
        return true
    else
        return false
    end
end

function ZI:GetPlayerInterrupts()
    for tbSpellPos, tbSpellId in pairs(ZI.InterruptSpells.SpellId) do
        if (IsPlayerSpell(tbSpellId)) then
            table.insert(ZI.PlayerInterrupts, tbSpellId)
            local tbCooldown = ZI.InterruptSpells.Cooldown[tbSpellPos]
            local tbSpellName = ZI.InterruptSpells.SpellName[tbSpellPos]
            --Debug("usable spell:" .. tbSpellName .. "," .. tostring(tbSpellId) .. "," .. tostring(tbCooldown))
            ZI:AddOrUpdateUnitInterrupt(ZI.PlayerName, tbSpellId, tbCooldown, tbSpellName)
        end
    end
    ZI:SendPlayerInterrupts()
end

function ZI:SendPlayerInterrupts()
    for _, spellId in pairs(ZI.PlayerInterrupts) do
        ZI:SendAddonMessage(ZI.PlayerName .. "," .. tostring(spellId))
    end
end

function ZI:UnitIsInParty(unitInterrupt)
    return unitInterrupt.IsPlayer or UnitInParty(unitInterrupt.UnitName)
end

function ZI:RemoveKickedPlayers(unitTable)
    local keysToRemove = {}
    for keyValue, unitInterrupt in pairs(unitTable) do
        if (not ZI:UnitIsInParty(unitInterrupt)) then
            table.insert(keysToRemove, keyValue)
        end
    end
    for _, keyValue in pairs(keysToRemove) do
        unitTable[keyValue] = nil
    end
end

function ZI:UpdateRoster()
    if (UnitInParty("player") or UnitInRaid("player")) then
        ZI.InGroup = true
    end
    ZI:RemoveKickedPlayers(ZI.UnitInterrupts)
    ZI:RemoveKickedPlayers(ZI.UnitInterruptOrder)
    ZI.UpdateInterruptRotation()
    --Debug(ZI.UnitInterrupts)
    --Debug(ZI.UnitInterruptOrder)
    ZI:SendPlayerInterrupts()
end

----------------------------------------
-- event handlers
----------------------------------------
-- PLAYER_LOGIN is the entry point of the main object. The PLAYER_LOGIN event is registered at the end of this file.
function ZI:PLAYER_LOGIN()
    if (not ZoomInterruptsDB) then
        ZoomInterruptsDB = {}
    end
    ZI.PlayerName = UnitName("player")
    ZI:SetInterruptSpells()
    ZI:GetPlayerInterrupts()
    C_ChatInfo.RegisterAddonMessagePrefix(addonName)

    -- register other events required by this addon
    self:RegisterEvent("CHAT_MSG_ADDON")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    self:RegisterEvent("GROUP_ROSTER_UPDATE")
    self:RegisterEvent("UNIT_FLAGS")

    ZI:CreateMainFrame()
end

function ZI:CHAT_MSG_ADDON(event, prefix, text, channel, sender, target, zoneChannelID, localID, name, instanceID)
    if (prefix == addonName) then
        ZI:AddonMessageReceived(text, channel, sender, target, zoneChannelID, localID, name, instanceID)
    end
end

function ZI:COMBAT_LOG_EVENT_UNFILTERED()
    ZI:CombatLogEvent(CombatLogGetCurrentEventInfo())
end

function ZI:GROUP_ROSTER_UPDATE()
    ZI:UpdateRoster()
end

function ZI:UNIT_FLAGS()
    ZI:UpdateInterruptRotation()
end

function ZI:SaveLayout(frame)
    if type(frame) ~= "string" then
        return
    end
    if not _G[frame] then
        return
    end

    local opt = ZoomInterruptsDB[frame] or nil

    if not opt then
        ZoomInterruptsDB[frame] = {
            ["point"] = "CENTER",
            ["relativePoint"] = "CENTER",
            ["xOfs"] = 0,
            ["yOfs"] = 0
        }
        opt = ZoomInterruptsDB[frame]
        return
    end

    local point, relativeTo, relativePoint, xOfs, yOfs = _G[frame]:GetPoint()
    opt.point = point
    opt.relativePoint = relativePoint
    opt.xOfs = xOfs
    opt.yOfs = yOfs
end

function ZI:RestoreLayout(frame)
    if type(frame) ~= "string" then
        return
    end
    if not _G[frame] then
        return
    end

    local opt = ZoomInterruptsDB[frame] or nil

    if not opt then
        ZoomInterruptsDB[frame] = {
            ["point"] = "CENTER",
            ["relativePoint"] = "CENTER",
            ["xOfs"] = 0,
            ["yOfs"] = 0
        }
        opt = ZoomInterruptsDB[frame]
    end

    _G[frame]:ClearAllPoints()
    _G[frame]:SetPoint(opt.point, UIParent, opt.relativePoint, opt.xOfs, opt.yOfs)
end

----------------------------------------
-- Register the entry point of the
--    main addon object
----------------------------------------
if IsLoggedIn() then
    ZI:PLAYER_LOGIN()
else
    ZI:RegisterEvent("PLAYER_LOGIN")
end
